#!/usr/bin/env python3

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: HY-menu
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# Small changes by Jani Jaakkola <jani.jaakkola@helsinki.fi>
# 18.10.2018 and ported to Python 3 by JJ 23.7.2020.

# Removing deprecated calls and migrate to emojis + reboot checks
# 31.05.2022 by N-V Koljonen

from gi.repository.AppIndicator3 import Indicator,\
    IndicatorCategory, IndicatorStatus
from gi.repository.Gio import FileMonitorEvent
import os
import platform
import subprocess

from menu import HYMenu, run_script, on_netdrive_click, error_notification
from computerinfo import computerinfo
from userinfo import userinfo
from printers import install_printers
from infowindow import info_window
from environment import ICON_PATH, URGENT_ICON_PATH

version = platform.freedesktop_os_release().get('VERSION_ID') \
                  .split('.')[0]


class IndicatorPlaces:
    def __init__(self):
        self.indicator = Indicator \
            .new("places",
                 ICON_PATH,
                 IndicatorCategory
                 .APPLICATION_STATUS)
        self.indicator \
            .set_attention_icon_full(URGENT_ICON_PATH,
                                     "HY Menu | Reboot required")
        self.update_computer_info()
        self.host_type = self.get_host_type()
        self.update_user_info()

        if self.host_type == "laptop":
            prefix = "sftp://melkki.cs.helsinki.fi/home/ad/"
        else:
            prefix = "/home/ad/"

        self.gpath = prefix + "P-drive"
        self.zpath = prefix + self.user_info["Username"][:1] \
            + "/" + self.user_info["Username"]
        self.remote_help_active = False

        self.handle_reboot()  # this also updates menu

    # Odd function signature allows using this as part of a button callback
    # without naming all the arguments
    def update_menu(self, *_):
        menu = HYMenu()

        menu.add_item(f"Cubbli {version}"),
        menu.add_item(self.computer_info["Computer name"],
                      icon=self.host_type,
                      action=self.open_computer_info)

        action = "Disable" if self.remote_help_active else "Enable"
        menu.add_item(f"{action} remote help",
                      icon="help",
                      action=self.remote_help)
        menu.add_separator()

        menu.add_bookmarks(self.host_type)

        if os.path.isfile('/etc/cubbli/upgrade-available'):
            menu.add_item("Upgrade Cubbli version",
                          icon="upgrade",
                          script="check-reinstall")
            menu.add_separator()

        menu.add_item("Computer Info",
                      icon="info",
                      action=self.open_computer_info)
        menu.add_item("User Info",
                      icon="user",
                      action=self.open_user_info)
        menu.add_item("HY Install Printers",
                      icon="printer",
                      action=open_printers)
        menu.add_separator()

        menu.add_item("IT Department group directories",
                      icon="drive",
                      action=on_netdrive_click,
                      arg=self.gpath)
        menu.add_item("Z-drive",
                      icon="drive",
                      action=on_netdrive_click,
                      arg=self.zpath)
        menu.add_item("OneDrive",
                      icon="drive",
                      script="cubbli-rclone-setup open")

        if self.reboot_required:
            menu.add_separator()
            menu.add_item("Reboot required!",
                          icon="reboot",
                          script="menu-reboot")  # FIXME

        menu.show_all()
        self.indicator.set_menu(menu.get_menu())

    def on_bookmarks_changed(self, filemonitor, file, other_file, event_type):
        if event_type == FileMonitorEvent.CHANGES_DONE_HINT:
            print('hy-menu: bookmarks changed, updating menu...')
            self.update_menu()

    def handle_reboot(self, *_):
        self.reboot_required = os.path.isfile("/var/run/reboot-required")
        if self.reboot_required:
            error_notification("Packages updated, reboot required")
            self.indicator.set_status(IndicatorStatus.ATTENTION)
        else:
            self.indicator.set_status(IndicatorStatus.ACTIVE)

        self.update_menu()

    # This functionality should be moved over to askforhelp.py
    def remote_help(self, _):
        activate = not self.remote_help_active
        action = "up" if activate else "down"

        for vpn in ["uh-vpn-allroute", "uh-vpn-remote-help"]:
            full_name = f"{vpn} ({self.user_info['Username']})"
            try:
                # Test VPN status, empty output = inactive VPN
                status = subprocess.check_output(["nmcli", "con", "show",
                                                 "--active", full_name],
                                                 timeout=15)

                # Enable/disable only when status differs from what is wanted
                if activate ^ bool(status):
                    run_script(["nmcli", "con", action, full_name])
            except TimeoutError:
                error_notification(f"{vpn} timeout after 15 seconds")
                return

        # Refresh for new IP address
        self.update_computer_info()

        if activate:
            d = dict()
            d["Username"] = self.user_info["Username"]
            d["Full name"] = self.user_info["Full name"]
            d["FQDN"] = self.computer_info["FQDN"]
            d["IP"] = self.computer_info["Remote help IP"]

            info_window("Remote help", d)

        self.remote_help_active = activate
        self.update_menu()

    def open_computer_info(self, _):
        self.update_computer_info()
        info_window("Computer info", self.computer_info)

    def open_user_info(self, _):
        self.update_user_info()
        info_window("User info", self.user_info)

    def update_user_info(self):
        self.user_info = userinfo()

    def update_computer_info(self):
        self.computer_info = computerinfo()

    def get_host_type(self):
        match self.computer_info["Computer name"][:2]:
            case "vd":
                return "vdi"
            case "lx":
                return "laptop"
            case _:
                return "desktop"  # Most likely this is the case


def open_printers(*_):
    install_printers()
