from gi.repository.Gtk import Clipboard, Button
from gi.repository.Gdk import SELECTION_CLIPBOARD

from itertools import repeat


# Convert a dictionary to clippable text.
# Keys are expected to be strings.
# If value is dictionary or list, they are formatted with respect to type,
# otherwise the value is converted to string.
def to_text(info):
    # Get width of the widest header
    header_width = max(map(lambda k: len(k), info.keys())) + 1

    items = []
    for k, v in info.items():
        if isinstance(v, dict):
            item = tabulated_dict_to_text(v)
        elif isinstance(v, list):
            item = "   ".join(v)
        else:
            item = str(v)

        if "\n" in item:
            item = multiline_padding(item, header_width)

        items.append(f"{k:<{header_width}}{item}")

    return '\n'.join(items)


# Add sufficient padding to all but first line of the text
# (first line is padded already)
def multiline_padding(text, header_width):
    rows = text.splitlines()
    padding = ''.join(repeat(" ", header_width))
    rows[1:] = list(map(lambda row: padding + row, rows[1:]))

    return '\n'.join(rows)


# Convert a dictionary to table with headers on top
def tabulated_dict_to_text(dict):
    # Make sure that every value is a list
    values = []
    for value in dict.values():
        if not isinstance(value, list):
            value = [value]
        values.append(value)

    widths = []
    # Convert each inner value to string
    columns = [[str(x) for x in list] for list in values]

    for i, header in enumerate(dict.keys()):
        # Prepend header to its column
        columns[i].insert(0, header)
        # Calculate maximum width of a column
        width = max([len(x) for x in columns[i]])
        widths.append(width)

    rows = []

    # Add column's padding for the field by fetching it from widths
    def format(col_item):
        column, item = col_item
        nonlocal widths

        return f"{item:<{widths[column]}}"

    # Iterate over rows
    for row in zip(*columns):
        # Format each element of a row
        r = map(format, enumerate(row))
        # Add each element to the final text
        rows.append(" ".join(r))

    return "\n".join(rows)


def set_clipboard(_, info):
    clipboard = Clipboard.get(SELECTION_CLIPBOARD)
    clipboard.set_text(to_text(info), -1)
    clipboard.store()


def clipboard_button(info):
    button = Button.new_with_label("Copy info to clipboard")
    button.connect("clicked", set_clipboard, info)
    button.show()

    return button
