#!/usr/bin/python3

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: Computer Info
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 23.3.2017


import subprocess
import os
import psutil
import socket
import re
import platform


class ComputerInfo:

    def __init__(self):
        # Class instance members
        self.computer_info_items = dict()

        self.create_computer_info()

    ############################
    # Computer info list items #
    ############################

    def set_computer_name_info(self):
        name = os.uname().nodename.split('.')[0]
        self.computer_info_items["Computer name"] = name

    def set_username_info(self):
        user = os.getlogin()
        self.computer_info_items["Username"] = user

    def set_cpu_name_info(self):
        cpu = "CPU name not found"
        with open("/proc/cpuinfo") as cpuinfo_file:
            for line in cpuinfo_file:
                if "model name" in line:
                    cpu = line.split(':')[1].strip()
                    # Break loop on first CPU name
                    break
        self.computer_info_items["CPU"] = cpu

    def set_ram_info(self):
        mem = int(psutil.virtual_memory().total / 1000000)
        self.computer_info_items["RAM (MB)"] = mem

    def set_ip_info(self):
        ips = []
        for ip in self.get_parsed_ip_addresses():
            if ip.startswith("10.115"):
                self.computer_info_items["Remote help IP"] = ip
            ips.append(ip)

        self.computer_info_items["IP addresses"] = ips

    def set_computer_fqdn_info(self):
        fqdn = os.uname().nodename
        self.computer_info_items["FQDN"] = fqdn

#    def set_mac_addres_info(self):
#        mac = self.get_ethernet_mac().upper()
#        self.computer_info_items["MAC address"] = mac

    def set_mac_addres_info(self):
        mac = self.get_ethernet_mac()
        if mac is not None:
            self.computer_info_items["MAC address"] = mac.upper()
        else:
            self.computer_info_items["MAC address"] = "MAC address not available"

    def set_serialnumber_info(self):
        serial = "serial not found"
        with open("/var/lib/cubbli/hostinfo", "r") as hostinfo_file:
            for line in hostinfo_file:
                if "DMI_chassis_serial" in line:
                    serial = line.split('=')[1].strip()
                    break
        self.computer_info_items["Serial number"] = serial

    def set_warranty_info(self):
        try:
            with open("/etc/cubbli/computer-warranty") as warranty_file:
                warranty = warranty_file.read().strip()
        except FileNotFoundError:
            warranty = "Not found"

        self.computer_info_items["Warranty expires"] = warranty

    def set_os_version_info(self):
        os_info = platform.freedesktop_os_release()
        os_info = f'{os_info["NAME"]} {os_info["VERSION"]}'
        self.computer_info_items["Operating system"] = os_info

    def set_last_update_info(self):
        last_update_time = subprocess.check_output("tail -1 /var/log/dpkg.log",
                                                   shell=True, text=True) \
            .split(" ")
        last_update_time = f'{last_update_time[0]} {last_update_time[1]}'
        self.computer_info_items["Last updated"] = last_update_time

    def set_local_drives_info(self):
        local_drives = subprocess.check_output("df -h -x tmpfs -x efivarfs",
                                               shell=True, text=True)
        self.computer_info_items["Local drives"] = parse_disk_free(local_drives)

    ################################
    # Generate computer info items #
    ################################

    # The ordering of the function calls here determines the ordering
    # of the info fields on the Gtk window
    def create_computer_info(self):
        self.set_computer_name_info()
        self.set_username_info()
        self.set_cpu_name_info()
        self.set_ram_info()
        self.set_ip_info()
        self.set_computer_fqdn_info()
        self.set_mac_addres_info()
        self.set_serialnumber_info()
        self.set_warranty_info()
        self.set_os_version_info()
        self.set_last_update_info()
        self.set_local_drives_info()
        return

    ###########################
    # Misc. parsing functions #
    ###########################

    # Get info on all available NICs
    # https://psutil.readthedocs.io/en/latest/index.html#psutil.net_if_addrs
    def get_computer_nic_info(self):
        return psutil.net_if_addrs().items()

    # Get MAC address of eth NIC
    def get_ethernet_mac(self):
        for interface, addresses in self.get_computer_nic_info():
            if re.search(r'.*enp.*', interface):
                for address in addresses:
                    if address.family == psutil.AF_LINK:
                        return address.address

    # Get all available IP addresses as a list
    def get_available_ips(self):
        ip_addresses = []
        for interface, addresses in self.get_computer_nic_info():
            for address in addresses:
                if address.family == socket.AF_INET:
                    ip_addresses.append(address.address)
        return ip_addresses

    # Parse the list of all available ipv4 addresses
    def get_parsed_ip_addresses(self):
        parsed_ips = []
        ips = self.get_available_ips()
        for ip in ips:
            if not (ip.startswith('127') or ip.startswith('192')):
                parsed_ips.append(ip)
        return sorted(parsed_ips)


# Convert tabulated df-data to a dictionary
def parse_disk_free(df_plaintext):
    lines = df_plaintext.splitlines()
    fields = list(map(lambda l: l.split(), lines))

    # Last two header columns are "Mounted" "on", merge them to one
    fields[0][-2] = f"{fields[0][-2]} {fields[0][-1]}"
    del fields[0][-1]

    # Transpose [[h0, h1], [v00, v01], [v10, v11]]
    # to        [[h0, v00, v10], [h1, v01, v11]]
    transposed = [list(i) for i in zip(*fields)]
    d = {column[0]: column[1:] for column in transposed}

    return d


def computerinfo():
    return ComputerInfo().computer_info_items
