from gi.repository.Gtk import Menu, MenuItem, SeparatorMenuItem
from gi.repository.Gio import app_info_launch_default_for_uri, \
    app_info_get_default_for_type
from gi.repository.Notify import Notification, init

import os
import subprocess
from subprocess import CalledProcessError

from environment import BOOKMARKS_PATH, URGENT_ICON_PATH


class HYMenu:
    def __init__(self):
        self.menu = Menu()
        self.icons = {
            "web":     "\U0001f30d",
            "drive":   "\U0001f4BE",
            "user":    "\U0001fa9e",
            "windows": "\U0001fa9f",
            "linux":   "\U0001f427",
            "printer": "\U0001f5A8",
            "help":    "\U0001F4F6",
            "laptop":  "\U0001f4BB",
            "desktop": "\U0001F5A5",
            "vdi":     "\U0001F310",
            "upgrade": "\U0001f503",
            "reboot":  "\U0001f534",
            "info":    "\U0001f4a1"
        }

    # Create and add an item to the menu with given label.
    # Action should be a function and arg is an argument to that function.
    # Shell is a path to some program.
    def add_item(self, label, icon=None, action=None, arg=None, script=None):
        item = MenuItem()

        # Use predefined icon or supplied icon if possible
        if (i := self.icons.get(icon)) or (i := icon):
            label = f"{i}\t{label}"
        item.set_label(label)

        if action:
            if arg:
                item.connect("activate", action, arg)
            else:
                item.connect("activate", action)

        if script:
            item.connect("activate", run_script, script)

        item.show()
        self.menu.append(item)

    # Add bookmarks to the menu from the /usr/share/hy-menu/hy-menu-items
    def add_bookmarks(self, host_type):
        with open(BOOKMARKS_PATH, "r") as bookmarks:
            for bm in bookmarks:
                path, label = bm.strip().partition(' ')[::2]
                icon = None

                if not label:
                    label = os.path.basename(os.path.normpath(path))

                if label.startswith('VDI C'):
                    icon = "linux"
                elif label.startswith('VDI W'):
                    icon = "windows"
                else:
                    icon = "web"

                # VDIs do not need VDI access
                if not (host_type == "vdi"
                        and path.startswith('vmware-view')):
                    self.add_item(label, icon=icon,
                                  action=on_bookmark_click,
                                  arg=path)

        self.add_separator()

    def add_separator(self):
        self.menu.append(SeparatorMenuItem())

    def get_menu(self):
        return self.menu

    def show_all(self):
        self.menu.show_all()


# Open network drive with the default file browser
# We redirect to directory, because file explorers can handle mounting
def on_netdrive_click(_, path):
    default = app_info_get_default_for_type('inode/directory', True)
    default.launch_uris([path])


# Open bookmark links with system default
def on_bookmark_click(_, path):
    app_info_launch_default_for_uri(path)


# This will run the *last* argument as a subprocess
# args-hack is used so that this can also be used as a callback
# Script can be a single string defining the command
# or a list containing program and arguments
def run_script(*args):
    script = args[-1]
    if not isinstance(script, list):
        script = script.split(' ')

    try:
        subprocess.run(script, text=True, check=True,
                       capture_output=True)
    except CalledProcessError as e:
        if error := e.stderr:
            error_notification(error)
        else:
            error_notification(f"Error running script {script}")


def error_notification(message):
    init("HY-menu")
    notification = Notification.new("HY-menu", message,
                                    URGENT_ICON_PATH)
    notification.show()
