#!/usr/bin/python3
# -*- coding: utf-8 -*-
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: HY install printers
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 23.3.2017
# JJ: ported to python 3

from gi.repository import Gtk
import subprocess
from base64 import b64decode


def fetch_printers():
    base = "OU=rps,OU=canon,OU=tulostus,OU=mvPalvelut,DC=ad,DC=helsinki,DC=fi"
    try:
        query = ["ldapsearch", "-o", "ldif-wrap=no", "-o", "nettimeout=3",
                 "-b", base, "objectClass=printQueue", "-LLL",
                 "location", "name"]
        res = subprocess.run(query, check=True, text=True, capture_output=True)

        printers = dict()

        # Each entry is separated with empty lines.
        # Last entry is empty
        for entry in res.stdout.split("\n\n")[:-1]:
            name, location = ("", "")
            for line in entry.splitlines():
                match line.split():
                    case ["name:", n]:
                        name = n
                    case ["location::", bytes]:  # Some lines are b64 encoded
                        location = b64decode(bytes).decode('utf-8')
                    case ["location:", l]:
                        location = l

                if name and location:
                    # Real name of printer is something like SPRUCE1-ab45.
                    server, name = name.split('-')
                    printers[name] = {"server": server,
                                      "location": location}

        return printers
    except Exception as e:
        print(e)


class InstallPrinter:
    def search_printer(self, _):
        self.listmodel.clear()
        text = self.entry.get_text()
        entry_text = text.lower().strip()

        for name, values in self.print_dict.items():
            location = values["location"]
            if entry_text in name.lower() or entry_text in location.lower():
                self.listmodel.append((name, location))

    def initialize_treeview(self):
        # create the liststore with 2 columns for strings
        self.listmodel = Gtk.ListStore(str, str)

        # create the treeview using liststore
        self.view = Gtk.TreeView(self.listmodel)
        # self.view.set_size_request(100, 320)

        # create the columns with text
        col1 = Gtk.TreeViewColumn('Serial number')
        col2 = Gtk.TreeViewColumn('Location')

        # add columns to treeview
        self.view.append_column(col1)
        self.view.append_column(col2)

        # create a CellRenderers to render the data
        cell1 = Gtk.CellRendererText()
        cell2 = Gtk.CellRendererText()

        # adds the cells to the columns
        col1.pack_start(cell1, True)
        col2.pack_start(cell2, True)

        # text=number chooses the text field from listmodel.append
        col1.set_attributes(cell1, text=0)
        col2.set_attributes(cell2, text=1)

        # make treeview searchable
        self.view.set_search_column(0)

        # enable column sorting
        col1.set_sort_column_id(0)
        col2.set_sort_column_id(1)

        # make treeview scrollable
        scrolled = Gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.view)

        for name, values in self.print_dict.items():
            location = values["location"]
            self.listmodel.append((name, location))

        return scrolled

    def install_selected_printer(self, _):
        selection = self.view.get_selection()
        treemodel, index = selection.get_selected_rows()
        row = treemodel[index]

        printer = row[0]
        location = row[1]
        server = self.print_dict[printer]["server"]

        popup = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL,
                                  Gtk.MessageType.QUESTION,
                                  Gtk.ButtonsType.YES_NO, None)
        popup.set_markup(f"Do you want to install {printer} in {location}?")
        added = False
        if popup.run() == Gtk.ResponseType.YES:
            path = "krbsmb://ATKK/" + server + ".ad.helsinki.fi/" + printer
            query = ["lpadmin", "-p", printer, "-v", path,
                     "-P", "/usr/share/ppd/cubbli/cel-iradvc5250-ps-en-jj.ppd",
                     "-E"]
            subprocess.run(query)
            added = True

        popup.destroy()

        if added:
            popup = Gtk.MessageDialog(None, Gtk.DialogFlags.MODAL,
                                      Gtk.MessageType.QUESTION,
                                      Gtk.ButtonsType.YES_NO, None)
            popup.set_markup("Set as default?")
            if popup.run() == Gtk.ResponseType.YES:
                subprocess.run(["lpadmin", "-d", printer])
            popup.destroy()

    def __init__(self):

        # get all the printers from AD to a variable
        self.print_dict = fetch_printers()

        # create the window
        app_window = Gtk.Window()
        app_window.set_title("HY Install Printers")

        # size of the window
        app_window.set_size_request(700, 700)
        # app_window.set_position(gtk.WIN_POS_CENTER)

        # create VBOX for the search field
        vbox = Gtk.Box.new(Gtk.Orientation.VERTICAL, 5)
        vbox.set_homogeneous(False)

        # create HBOX for the buttons
        buttons = Gtk.Box.new(Gtk.Orientation.HORIZONTAL, 5)

        # create the search field with max lenght and preset text
        self.entry = Gtk.Entry()
        self.entry.set_max_length(72)
        self.entry.connect("activate", self.search_printer)
        self.entry.set_placeholder_text("Printer serial number or location")

        label = Gtk.Label.new("Printer serial number or location")
        label.set_xalign(0)
        vbox.add(label)

        vbox.add(self.entry)

        # create the "search" button
        button1 = Gtk.Button("Search")
        button1.connect("released", self.search_printer)
        buttons.pack_start(button1, False, False, 0)

        # create the "add" button
        button2 = Gtk.Button("Add printer")
        button2.connect("released", self.install_selected_printer)
        buttons.pack_start(button2, False, False, 0)

        # add buttons to the VBOX
        vbox.add(buttons)

        # add treeview to VBOX
        vbox.pack_start(self.initialize_treeview(), True, True, 0)

        # add VBOX to the WINDOW
        app_window.add(vbox)

        # show all
        app_window.show_all()
        return


def install_printers():
    InstallPrinter()
