#!/usr/bin/python3

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: User Info
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 27.4.2017

# Refactored by Mikko Halmela in July 2024

import subprocess
import os
import datetime
from itertools import takewhile

# Lookup dictionary for values wanted from LDAP
# Key is LDAP-value, value is the display name
wanted = {
          "displayName":                         "Full name",
          "mail":                                "Email",
          "pwdLastSet":                          "Password changed",
          "msDS-UserPasswordExpiryTimeComputed": "Password expires",
          "accountExpires":                      "Account expires",
          "msExchRecipientTypeDetails":          "Mailbox",
      }


class UserInfo:
    def __init__(self):
        self.parse_info()

    def create_info(self):
        self.username = os.getlogin()
        arguments = ["ldapsearch", "-LLL", "-o", "nettimeout=3",
                     f"cn={self.username}"]
        query = arguments + list(wanted.keys())
        try:
            userinfo = subprocess.run(query, check=True, text=True,
                                      capture_output=True).stdout
            # Skip the first line containing general info
            userinfo = userinfo.splitlines()[1:]
            # Take only rows containing our info (stop after first empty line)
            userinfo = takewhile(lambda x: x, userinfo)
            # ['wanted1: field1', 'wanted2: field2'...]
            # -> [['wanted1', 'field1'], ['wanted2', 'field2']...]
            userinfo = map(lambda x: x.split(": "), userinfo)
            # {'header1': 'field1', 'header2': 'field2'..}
            userinfo = map(lambda x: (wanted[x[0]], x[1]), userinfo)

            return dict(userinfo)
        except Exception:
            print("FAILED")  # TODO
            return None

    def parse_info(self):
        raw_info = self.create_info()
        self.ready_info = {}
        self.ready_info["Username"] = self.username

        if not raw_info:
            return

        self.ready_info["Full name"] = raw_info["Full name"]
        self.ready_info["Email"] = raw_info["Email"]

        self.ready_info["Password changed"] = \
            calculate_date(raw_info["Password changed"])
        self.ready_info["Password expires"] = \
            calculate_date(raw_info["Password expires"])
        self.ready_info["Account expires"] = \
            calculate_date(raw_info["Account expires"])

        # M$ magic number
        self.ready_info["Mailbox"] = "Online (Cloud)" \
            if raw_info["Mailbox"] == "2147483648" \
            else "Local"

        default_printer = subprocess.check_output("lpstat -d",
                                                  shell=True, text=True) \
            .strip()
        if default_printer.startswith("no"):
            self.ready_info["Default printer"] = "Not set"
        else:
            self.ready_info["Default printer"] = default_printer.split(": ")[1]


# Microsoft's epoch is 1601-01-01 so we have to calculate offset
def calculate_date(time):
    time = datetime.timedelta(seconds=int(time) / 10000000)
    ansistandard = datetime.datetime(1601, 1, 1)
    return (ansistandard+time).strftime("%a, %d %b %Y %H:%M:%S")


def userinfo():
    return UserInfo().ready_info
